@@ -7,6 +7,7 @@ from __future__ import annotations
 import glob
 import itertools
 import json
+import os
 import pathlib
 import re
 import sys
@@ -14,10 +15,10 @@ import uuid
 
 import nox
 
-try:
+if sys.version_info >= (3, 11):
     import tomllib
-except ImportError:
-    import tomli as tomllib  # type: ignore[import-not-found,no-redef]
+else:
+    import tomli as tomllib
 
 nox.options.reuse_existing_virtualenvs = True
 nox.options.default_venv_backend = "uv|virtualenv"
@@ -59,10 +60,11 @@ def tests(session: nox.Session) -> None:
         pathlib.Path(".") / ".rust-cov" / str(uuid.uuid4())
     ).absolute()
     if session.name != "tests-nocoverage":
+        rustflags = os.environ.get("RUSTFLAGS", "")
+        assert rustflags is not None
         session.env.update(
             {
-                "RUSTFLAGS": "-Cinstrument-coverage "
-                + session.env.get("RUSTFLAGS", ""),
+                "RUSTFLAGS": f"-Cinstrument-coverage {rustflags}",
                 "LLVM_PROFILE_FILE": str(prof_location / "cov-%p.profraw"),
             }
         )
@@ -86,6 +88,7 @@ def tests(session: nox.Session) -> None:
         cov_args = [
             "--cov=cryptography",
             "--cov=tests",
+            "--cov-context=test",
         ]
     else:
         cov_args = []
@@ -213,15 +216,15 @@ def flake(session: nox.Session) -> None:
 
 
 @nox.session
-@nox.session(name="rust-noclippy")
 def rust(session: nox.Session) -> None:
     prof_location = (
         pathlib.Path(".") / ".rust-cov" / str(uuid.uuid4())
     ).absolute()
+    rustflags = os.environ.get("RUSTFLAGS", "")
+    assert rustflags is not None
     session.env.update(
         {
-            "RUSTFLAGS": "-Cinstrument-coverage  "
-            + session.env.get("RUSTFLAGS", ""),
+            "RUSTFLAGS": f"-Cinstrument-coverage  {rustflags}",
             "LLVM_PROFILE_FILE": str(prof_location / "cov-%p.profraw"),
         }
     )
@@ -232,16 +235,15 @@ def rust(session: nox.Session) -> None:
     install(session, *pyproject_data["build-system"]["requires"])
 
     session.run("cargo", "fmt", "--all", "--", "--check", external=True)
-    if session.name != "rust-noclippy":
-        session.run(
-            "cargo",
-            "clippy",
-            "--all",
-            "--",
-            "-D",
-            "warnings",
-            external=True,
-        )
+    session.run(
+        "cargo",
+        "clippy",
+        "--all",
+        "--",
+        "-D",
+        "warnings",
+        external=True,
+    )
 
     build_output = session.run(
         "cargo",
@@ -271,7 +273,7 @@ def rust(session: nox.Session) -> None:
 
 
 @nox.session
-def local(session):
+def local(session: nox.Session):
     pyproject_data = load_pyproject_toml()
     install(session, "-e", "./vectors", verbose=False)
     install(
@@ -345,11 +347,6 @@ def process_rust_coverage(
     rust_binaries: list[str],
     prof_raw_location: pathlib.Path,
 ) -> None:
-    # Hitting weird issues merging Windows and Linux Rust coverage, so just
-    # say the hell with it.
-    if sys.platform == "win32":
-        return
-
     target_libdir = session.run(
         "rustc", "--print", "target-libdir", external=True, silent=True
     )
