@@ -5,6 +5,7 @@
 
 import pytest
 
+from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric.utils import (
     Prehashed,
     decode_dss_signature,
@@ -34,25 +35,27 @@ def test_dss_signature():
 
 
 def test_encode_dss_non_integer():
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         encode_dss_signature("h", 3)  # type: ignore[arg-type]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         encode_dss_signature("3", "2")  # type: ignore[arg-type]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         encode_dss_signature(3, "h")  # type: ignore[arg-type]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         encode_dss_signature(3.3, 1.2)  # type: ignore[arg-type]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         encode_dss_signature("hello", "world")  # type: ignore[arg-type]
 
 
 def test_encode_dss_negative():
     with pytest.raises(ValueError):
         encode_dss_signature(-1, 0)
+    with pytest.raises(ValueError):
+        encode_dss_signature(0, -1)
 
 
 def test_decode_dss_trailing_bytes():
@@ -74,3 +77,8 @@ def test_decode_dss_invalid_asn1():
 def test_pass_invalid_prehashed_arg():
     with pytest.raises(TypeError):
         Prehashed(object())  # type: ignore[arg-type]
+
+
+def test_prehashed_digest_size():
+    p = Prehashed(hashes.SHA256())
+    assert p.digest_size == 32
